"""Rule objects for approval system."""
import logging

import InvenTree.helpers
from plugin import registry as plugin_registry

from .ApprovalPlugin import ApprovalPlugin
from InvenTree.config import do_typecast

logger = logging.getLogger("inventree")


class ApprovalRule:
    """Base class for notification methods."""

    NAME = ""
    DESCRIPTION = None
    IDENTIFIER = None
    SETTING = None

    def settings_value(self, cast=None):
        """Return the value of the setting."""
        if self.SETTING is None:
            return None
        value = plugin_registry.get_plugin(ApprovalPlugin.SLUG).get_setting(
            self.SETTING
        )
        if cast:
            value = do_typecast(value, cast)
            if cast == int and value == "":
                value = 0
        return value

    def check(self, approval, target, decisions: list):
        """Check if the rule is fulfilled.

        Returns:
            bool: True if the approval is accepted, False if the approval is rejected, None if unconclusive.
        """
        raise NotImplementedError("A ApprovalRule must define a `check` method")


class RuleRegistryClass:
    """Class that acts as registry for all available approval rule definitions."""

    items = None

    def collect(self):
        """Collect all classes in the environment that are approval rule definitions."""
        logger.debug("Collecting approval rule definitions")
        rules = InvenTree.helpers.inheritors(ApprovalRule)
        self.items = {x.IDENTIFIER: x for x in rules}
        logger.debug(f"Found {len(registry.items)} approval rule definitions")


registry = RuleRegistryClass()
